#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import re
import alfred
from subprocess import Popen, PIPE, call


def arg(pos):
    try:
        args = sys.argv[1].split()
    except:
        args = []
    try:
        return args[pos]
    except:
        return ''


def is_email(email):
    if re.match(r"[^@]+@[^@]+\.[^@]+", email):
        return True
    else:
        return False


def frontmost_app_name():
    scpt = 'tell application "System Events"\n'\
        'set current_app to first item of '\
        '(get name of processes whose frontmost is true)\n'\
        'return current_app\n'\
        'end tell'
    p = Popen(['osascript'], stdin=PIPE, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate(scpt)
    if p.returncode == 0:
        return stdout.strip()
    else:
        return None


def frontmost_app_path():
    scpt = 'set app_path to POSIX path of (path to frontmost application)\n'\
        'return app_path'
    p = Popen(['osascript'], stdin=PIPE, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate(scpt)
    if p.returncode == 0:
        return stdout.strip()
    else:
        return None


def dialog(msg, placeholder='', hide_input=False, is_choice=False):
    if hide_input:
        cmd = 'with hidden answer'
    else:
        cmd = ''
    if is_choice:
        scpt = '''
            display dialog "{}" buttons {{"No", "Yes"}} default button "Yes"
            '''.format(msg)
    else:
        scpt = 'set input_text to text returned of '\
            '(display dialog "{}" default answer "{}" {})\n'\
            'return input_text'.format(msg, placeholder, cmd)

    p = Popen(['osascript'], stdin=PIPE, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate(scpt)
    if p.returncode == 0:
        return stdout.strip()
    else:
        return None


def add_to_keychain(account, password):
    p = Popen(['/usr/bin/security',
               'add-generic-password',
               '-a', account,
               '-s', 'Apple Account Switcher',
               '-w', password,
               '-T', ''], stdin=PIPE, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate()
    return p.returncode


def delete_from_keychain(account):
    p = Popen(['/usr/bin/security',
               'delete-generic-password',
               '-a', account,
               '-s', 'Apple Account Switcher'],
              stdin=PIPE, stdout=PIPE, stderr=PIPE)
    stdout, stderr = p.communicate()
    return p.returncode


def menu_switch(keyword):
    keyword = keyword.lower()

    app_name = frontmost_app_name()
    if app_name not in ('iTunes', 'iBooks', 'App Store'):
        app_name = 'iTunes'
    icon = app_name + '.png'

    accounts = alfred.config.get('accounts')
    if accounts is None:
        accounts = []

    feedback = alfred.Feedback()
    for account in accounts:
        account_id, account_title, is_match_on = account
        if keyword not in account_title.lower()\
            and keyword not in account_id.lower()\
                and keyword != '':
            continue
        if is_match_on and app_name == 'iTunes':
            txt = ', and turn on iTunes Match'
        else:
            txt = ''
        feedback.addItem(
            title=account_title,
            subtitle='Switch {} account to {}{}'.format(
                app_name, account_id, txt),
            icon=icon,
            arg='{} {}'.format(account_id, is_match_on)
        )

    if keyword in 'settings' or keyword == '':
        feedback.addItem(
            title='Settings',
            subtitle='To add or delete accounts',
            icon="settings.png",
            autocomplete='settings',
            valid=False,
        )
    if feedback.isEmpty():
        feedback.addItem(
            title='No match found',
            subtitle='',
            autocomplete='',
            valid=False,
        )
    feedback.output()


def menu_fill(keyword):
    keyword = keyword.lower()

    app_name = frontmost_app_name()
    icon = frontmost_app_path()

    accounts = alfred.config.get('accounts')
    if accounts is None:
        accounts = []

    feedback = alfred.Feedback()
    for account in accounts:
        account_id, account_title, _ = account
        if keyword not in account_title.lower()\
            and keyword not in account_id.lower()\
                and keyword != '':
            continue
        feedback.addItem(
            title=account_title,
            subtitle='Autofill the password of {} to {}'.format(
                account_id, app_name),
            icon=icon,
            icontype="fileicon",
            arg='{}'.format(account_id)
        )

    if keyword in 'settings' or keyword == '':
        feedback.addItem(
            title='Settings',
            subtitle='To add or delete accounts',
            icon="settings.png",
            autocomplete='settings',
            valid=False,
        )
    if feedback.isEmpty():
        feedback.addItem(
            title='No match found',
            subtitle='',
            autocomplete='',
            valid=False,
        )
    feedback.output()


def menu_settings():
    accounts = alfred.config.get('accounts')
    if accounts is None:
        accounts = []

    feedback = alfred.Feedback()
    feedback.addItem(
        title='Add an Apple account',
        subtitle='',
        arg='settings add'
    )
    for account in accounts:
        account_id, account_title, _ = account
        feedback.addItem(
            title='Delete account \'{}\''.format(account_id),
            subtitle=account_title,
            arg='settings delete {}'.format(account_id)
        )
    feedback.addItem(
        title='Delete all accounts',
        subtitle='',
        arg='settings deleteall'
    )
    feedback.output()


def switch_account(account_id, is_match_on):
    if is_match_on == '':
        accounts = alfred.config.get('accounts')
        for account in accounts:
            if account[0] == account_id:
                is_match_on = str(account[2])
                break
    accounts = alfred.config.get('accounts')
    call(['osascript', 'switch.scpt', account_id, is_match_on])


def fill_password(account_id):
    call(['osascript', 'fill.scpt', account_id])


def add_account():
    account_id = dialog('Please enter your Apple account',
                        'example@icloud.com')
    if account_id is None:
        alfred.exit('Failed to add account')
    if not is_email(account_id):
        alfred.exit('Invalid email address')
    account_password = dialog('Please enter your password\n'
                              '(It will only be stored in Keychain)', '', True)
    if account_password is None:
        alfred.exit('Failed to add account')
    if account_password is '':
        alfred.exit('Blank password')
    account_title = dialog(
        'Please enter a title to show in Alfred', 'China Account')
    if account_title is None or account_title == '':
        account_title = account_id
    button_result = dialog(
        'Do you want to turn on iTunes Match automatically?', is_choice=True)
    if button_result == 'button returned:Yes':
        is_match_on = True
    else:
        is_match_on = False

    result = add_to_keychain(account_id, account_password)
    if result != 0:
        alfred.exit('Failed to add account to Keychain')

    accounts = alfred.config.get('accounts')
    if accounts is None:
        accounts = []
    accounts.append((account_id, account_title, is_match_on))
    alfred.config.set(accounts=accounts)

    alfred.exit('Added account \'{}\' to Keychain'.format(account_id))


def delete_account(account_id):
    accounts = alfred.config.get('accounts')
    for account in accounts:
        if account[0] == account_id:
            accounts.remove(account)
            break
    alfred.config.set(accounts=accounts)
    result = delete_from_keychain(account_id)
    if result != 0:
        alfred.exit(
            'Failed to delete account \'{}\' from Keychain').format(account_id)

    alfred.exit('Deleted account \'{}\' from Keychain'.format(account_id))


def delete_all():
    accounts = alfred.config.get('accounts')
    if accounts is None:
        accounts = []
    count = len(accounts)
    for account in accounts:
        account_id = account[0]
        delete_from_keychain(account_id)
    alfred.config.set(accounts=[])

    alfred.exit('Deleted {} account(s) from Keychain'.format(count))


def main():
    if arg(1) == 'settings':
        if arg(2) == 'add':
            add_account()
        elif arg(2) == 'delete':
            delete_account(arg(3))
        elif arg(2) == 'deleteall':
            delete_all()
        else:
            menu_settings()
    elif is_email(arg(1)):
        if arg(0) == 'switch':
            switch_account(arg(1), arg(2))
        elif arg(0) == 'fill':
            fill_password(arg(1))
    else:
        if arg(0) == 'switch':
            menu_switch(arg(1))
        elif arg(0) == 'fill':
            menu_fill(arg(1))


if __name__ == "__main__":
    main()
